import vtk
import os


def get_program_parameters():
    import argparse
    description = 'Force Velocity zero at Wing.'
    epilogue = ''''''
    parser = argparse.ArgumentParser(description=description, epilog=epilogue,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('filename', help='CellCentered.case.')
    args = parser.parse_args()
    return args.filename


def main():

    filename = get_program_parameters()
    case_file_name = os.path.basename(filename)
    path = os.path.dirname(filename)

    if path == '':
        path = './'

    print(case_file_name, path)

    reader = vtk.vtkGenericEnSightReader()
    reader.SetFilePath(path)
    reader.SetCaseFileName(case_file_name)
    reader.ReadAllVariablesOn()
    reader.UpdateInformation()
    reader.Update()

    # Extract fluid zone
    input = vtk.vtkUnstructuredGrid.SafeDownCast(reader.GetOutput().GetBlock(0))

    point_locator = vtk.vtkPointLocator()
    point_locator.SetDataSet(input)
    point_locator.BuildLocator()

    c_to_p = vtk.vtkCellDataToPointData()
    c_to_p.SetInputData(input)
    c_to_p.PassCellDataOff()
    c_to_p.Update()

    output = vtk.vtkUnstructuredGrid.SafeDownCast(c_to_p.GetOutput())

    extract_surface = vtk.vtkGeometryFilter()
    extract_surface.SetInputData(output)

    normal_generator = vtk.vtkPolyDataNormals()
    normal_generator.SetInputConnection(extract_surface.GetOutputPort())
    normal_generator.SetFeatureAngle(60)
    normal_generator.SplittingOn()
    normal_generator.ConsistencyOn()

    connectivity = vtk.vtkConnectivityFilter()
    connectivity.SetInputConnection(normal_generator.GetOutputPort())
    connectivity.SetExtractionModeToClosestPointRegion()
    connectivity.SetClosestPoint(0.00693951, -0.00351305, 0.0302107)
    connectivity.Update()

    velocity = output.GetPointData().GetArray("velocity")

    wing_points = connectivity.GetOutput().GetPoints()

    for pt_id in range(wing_points.GetNumberOfPoints()):
        point = wing_points.GetPoint(pt_id)
        id = point_locator.FindClosestPoint(point)

        velocity.SetTuple3(id, 0.0, 0.0, 0.0)

    writer = vtk.vtkXMLUnstructuredGridWriter()
    writer.SetFileName('fluid.vtu')
    writer.SetInputData(output)
    writer.Write()


if __name__ == '__main__':
    main()
